@@ -1,8 +1,10 @@
+import os
 import urllib3
 import functools
 import platform
 import subprocess
 from log import log
+from git import Repo,GitCommandError
 
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
@@ -62,4 +64,43 @@ class Server():
         except subprocess.CalledProcessError as e:
             log.warning(f"failed to run command {command},stderr is {e.stderr}, exit_code is {e.returncode}")
             rs = ExecResult(stdout=e.stdout, stderr=e.stderr, exit_code=e.returncode)
-            return rs
\ No newline at end of file
+            return rs
+
+    @enter_and_leave_function
+    def git_clone(self,repo_url,branch,dest_path):
+        try:
+            if os.path.exists(dest_path):
+                if os.path.isdir(os.path.join(dest_path, '.git')):
+                    log.warn(f"dest_path '{dest_path}' already exist and it is git repos.")
+                    return False
+                else:
+                    log.warn(f"dest_path '{dest_path}' already exist but it is not git repos.")
+                    return False
+
+            parent_dir = os.path.dirname(dest_path)
+            if parent_dir and not os.path.exists(parent_dir):
+                os.makedirs(parent_dir, exist_ok=True)
+
+            repo = Repo.clone_from(
+                url=repo_url,
+                to_path=dest_path,
+                branch=branch
+            )
+
+            if repo.bare:
+                log.warn("failed to git clone but create empty dir")
+                return False
+
+            current_branch = repo.active_branch.name
+            if current_branch != branch:
+                log.warn(f"git clone successful but branch is: {current_branch}, expected: {branch}")
+                return False
+            log.info(f"git clone successful dest_path: {dest_path}\ncurrent branch: {current_branch}")
+            return True
+
+        except GitCommandError as e:
+            log.warn(f"failed to run git command: {str(e)}")
+            return False
+        except Exception as e:
+            log.warn(f"error : {str(e)}")
+            return False
\ No newline at end of file
